#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <algorithm>
#include <cassert>

using namespace std;

const int N = 50;
const int MOD = 1e9 + 7;

int add(int a, int b) {
    if (a + b < MOD) {
        return a + b;
    }
    return a + b - MOD;
}

int sub(int a, int b) {
    if (a >= b) {
        return a - b;
    }
    return a - b + MOD;
}

int mul(int a, int b) {
    return 1LL * a * b % MOD;
}

int binpow(int a, int b) {
    int res = 1;
    while (b) {
        if (b & 1) {
            res = mul(res, a);
        }
        a = mul(a, a);
        b >>= 1;
    }
    return res;
}

struct State {
    int i;
    vector<int> cup;
    vector<int> cdown;
    vector<int> badu;
    vector<int> badd;

    State() {
        i = 0;
        cup = cdown = badu = badd = {0, 0};
    }

    State(int i_, vector<int> cup_, vector<int> cdown_, vector<int> badu_, vector<int> badd_) {
        i = i_;
        cup = cup_;
        cdown = cdown_;
        badd = badd_;
        badu = badu_;
    }

    State add(int f, int s) {
        vector<int> ncup = cup;
        vector<int> ndown = cdown;
        vector<int> nbadu = badu;
        vector<int> nbadd = badd;
        ncup[f]++;
        ndown[s]++;
        nbadu[f] = 0;
        nbadd[s] = 0;
        if (f != s && cup[f] == 0) {
            nbadu[f] = 1;
        }
        if (f != s && cdown[s] == 0) {
            nbadd[s] = 1;
        }
        // cerr << f << ' ' << s << ' ' << cup[f] << ' ' << cdown[s] << endl;
        return State(i + 1, ncup, ndown, nbadu, nbadd);
    }

    bool ok() {
        return badd[0] == 0 && badd[1] == 0 && badu[0] == 0 && badu[1] == 0;
    }

    bool operator<(const State &other) const {
        return make_tuple(i, cup, cdown, badu, badd) < make_tuple(other.i, other.cup, other.cdown, other.badu, other.badd);
    }
};

int res[N];
vector<int> out;

map<State, int> dp;

void init_dp() {
    // State cur;
    // cur = cur.add(0, 1);
    // cout << cur.ok() << endl;

    dp[State()] = 1;
    set<State> S;
    S.insert(State());
    set<State> poped;
    while (!S.empty()) {
        State cur = *S.begin();
        S.erase(*S.begin());
        // assert(!poped.count(cur));
        // poped.insert(cur);
        cerr << "pop " << cur.i << endl;
        for (int f = 0; f < 2; f++) {
            for (int s = 0; s < 2; s++) {
                State nw = cur.add(f, s);
                if (nw.i >= N) {
                    continue;
                }
                if (!dp.count(nw)) {
                    S.insert(nw);
                }
                dp[nw] = add(dp[nw], dp[cur]);
            }
        }
    }
    for (auto [cur, s] : dp) {
        if (cur.badu[0] == 0 && cur.badu[1] == 0 && cur.badd[0] == 0 && cur.badd[1] == 0) {
            res[cur.i] = add(res[cur.i], s);
        }
    }
    for (int i = 0; i < N; i++) {
        cout << res[i] << ", ";
    }
    cout << endl;
    exit(0);
}


int f[N];
int C[N][N];

void init() {
    for (int i = 0; i < N; i++) {
        C[i][0] = 1;
        C[i][i] = 1;
        for (int j = 1; j < i; j++) {
            C[i][j] = add(C[i - 1][j], C[i - 1][j - 1]);
        }
    }
    f[0] = 1;
    for (int i = 1; i < N; i++) {
        f[i] = mul(f[i - 1], i);
    }
}

int n, m;

int cnt(int x) {
    int def = 2;
    def = mul(def, C[n][x]);
    def = mul(def, C[m][x]);
    def = mul(def, f[x]);
    int fr = 1;
    {
        int free = (n * m) - ((n + m) * x - x * x);
        fr = mul(fr, binpow(2, free));
    }
    return mul(def, fr);
}

signed main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);

    init();
    // init_dp();
    out = {1, 2, 6, 34, 160, 754, 3400, 14690, 61568, 253090, 1028296, 4149490, 16679200, 66896210, 267977096, 72759227, 292870628, 175413195, 710039724, 857982804, 469677843, 958400823, 1372864, 357810178, 169435122, 221240988, 106185748, 135625393, 501140922, 995588136, 111889312, 1602679, 704441037, 393699093, 86410041, 88349778, 277777241, 837777027, 560261454, 170980230, 567037987, 80874884, 41915028, 790421801, 779063006, 94699125, 823071821, 155589655, 298456022, 444992714};

    cin >> n >> m;
    if (n < m) swap(n, m);
    if (m == 1) {
        auto ok = [&](int cnt) {
            return cnt != 1;
        };
        int ans = 0;
        for (int cw = 0; cw <= n; cw++) {
            if (ok(cw) && ok(n - cw)) {
                ans = add(ans, C[n][cw]);
            }
        }
        cout << ans << '\n';
        return 0;
    }
    if (m == 2) {
        cout << out[n] << '\n';
        return 0;
    }
    int ans = binpow(2, n * m);
    int bad = 0;
    for (int c = 1; c <= min(n, m); c++) {
        if (c % 2 == 1) {
            bad = add(bad, cnt(c));
        } else {
            bad = sub(bad, cnt(c));
        }
    }
    cout << sub(ans, bad) << '\n';
}
