#include "sqsort.h"
#include <vector>
#include <algorithm>
#include <cassert>
#include <iostream>
#include <random>
#include <chrono>
#include <set>

std::vector<int> id;
std::vector<int> orb;

bool compare(std::pair<int,int> f1, std::pair<int,int> f2) {
  if(f1.second < f1.first)
    std::swap(f1.first, f1.second);
  if(f2.second < f2.first)
    std::swap(f2.second, f2.first);
  std::pair<int, int> fs1 = f1, fs2 = f2;
  if(id[fs1.first] > id[fs1.second])
    std::swap(fs1.first, fs1.second);
  if(id[fs2.first] > id[fs2.second])
    std::swap(fs2.first, fs2.second);
   
  if(id[fs1.first] <= id[fs2.first] && id[fs1.second] <= id[fs2.second]){
    return 1;
  }
  if(id[fs1.first] >= id[fs2.first] && id[fs1.second] >= id[fs2.second]) {
    return 0;
  }
  
  return cmp(f1.first, f1.second, f2.first, f2.second);
}

std::mt19937 rng(std::chrono::high_resolution_clock::now().time_since_epoch().count());

int random(int lim) {
  return std::uniform_int_distribution<int>(0, lim)(rng);
}

bool compare2(int val, int val2) {
  return cmp(val, val, val2, val2);
}

void precompute(int n) {
  id.resize(n);
  orb.resize(n);
  for(int i = 0; i < n; i++)
    orb[i] = i;

  std::sort(orb.begin(), orb.end(), compare2);
  for(int i = 0;i < n; i++)
    id[orb[i]] = i;
}

std::vector<std::pair<int,int>> solve(int n) {

  precompute(n);

  std::vector<std::vector<std::pair<int,int>>> ord;
  ord.resize(n);
  int l = (orb.size() + 1) / 2;
  for(int i = 0; i < l; i++) {
    int x = i;
    int y = i;
    for(int j = i; j < orb.size() - i; j++, y++)  {
      ord[i].push_back({orb[x], orb[y]});
    }
    y--;
    x++;
    for(int j = i; j < orb.size() - i - 1; j++, x++) {
      ord[i].push_back({orb[x], orb[y]});
    }
   }

   std::set<std::pair<int,int>> myset;
   
   for(int i = 0; i < l; i++)
     myset.insert({ord[i].size(), i});

   while(1 < myset.size()) {
     int x = myset.begin()->second;
     myset.erase(myset.begin());
     int y = myset.begin()->second;
     myset.erase(myset.begin());
     
     std::vector<std::pair<int,int>> aux(ord[x].size() + ord[y].size());
     std::merge(ord[x].begin(), ord[x].end(), ord[y].begin(), ord[y].end(), aux.begin(), compare);
     ord[x] = aux;
     myset.insert({ord[x].size(), x});
  }

  return ord[myset.begin()->second];
}
