//
// Created by Dmitry Dubrovin on 11/20/20.
//

#include "sqsort.h"
#include <vector>
#include <algorithm>
#include <map>

using namespace std;

bool comp(pair<int, int> a, pair<int, int> b) {
    return cmp(a.first, a.second, b.first, b.second);
}

vector<int> indexes;


vector<pair<int, int>> buffer;

int mless(int a, int b) {
    return indexes[a] < indexes[b];
}

bool true_comp(pair<int, int> a, pair<int, int> b) {
    if (!mless(a.first, a.second)) {
        swap(a.first, a.second);
    }
    if (!mless(b.first, b.second)) {
        swap(b.first, b.second);
    }
    if (mless(a.first, b.first) && mless(a.second, b.second)) {
        return true;
    }
    if (!mless(a.first, b.first) && !mless(a.second, b.second)) {
        return false;
    }
    return comp(a, b);
}

void mrg(vector<pair<int, int>> &vec, int l1, int r1, int l2, int r2, bool cmp) {
    int s = l1;
    while (l1 < r1 && l2 < r2) {
        bool cm = 0;
        if (cmp) {
            cm = true_comp(vec[l1], vec[l2]);
        } else {
            cm = comp(vec[l1], vec[l2]);
        }
        if (cm) {
            buffer.push_back(vec[l1++]);
        } else {
            buffer.push_back(vec[l2++]);
        }
    }
    while (l1 < r1) {
        buffer.push_back(vec[l1++]);
    }
    while (l2 < r2) {
        buffer.push_back(vec[l2++]);
    }
    for (int i = 0; i < buffer.size(); ++i) {
        vec[s + i] = buffer[i];
    }
    buffer.clear();
}

void msort(vector<pair<int, int>> &vec, int l, int r, bool cmp) {
    if (r == l + 1) {
        return;
    }
    int m = (r + l) / 2;
    msort(vec, l, m, cmp);
    msort(vec, m, r, cmp);
    mrg(vec, l, m, m, r, cmp);
}


std::vector<std::pair<int, int>> solve(int n) {
    vector<pair<int, int>> tmp(n);
    for (int i = 0; i < n; ++i) {
        tmp[i] = {i, i};
    }
    msort(tmp, 0, n, 0);
    indexes.resize(n);
    for (int i = 0; i < n; ++i) {
        indexes[tmp[i].first] = i;
    }
    vector<pair<int, int>> res;
    for (int i = 0; i < n; ++i) {
        for (int j = i; j < n; ++j) {
            res.push_back({i, j});
        }
    }
    msort(res, 0, res.size(), 1);
    return res;
}
