#include <iostream>
#include <string>
#include <vector>
#include <queue>
#include <deque>
#include <stack>
#include <set>
#include <map>
#include <unordered_map>
#include <unordered_set>
#include <cstring>
#include <cmath>
#include <cstdlib>
#include <algorithm>
#include <random>
#include <iomanip>
#include <functional>
#include <cassert>
#include <bitset>
#include <chrono>
 
using namespace std;
 
typedef long long ll;

#include "sqsort.h"

vector <int> p;
vector <vector <int>> pos;


struct ccmp {
  bool operator()(pair <int, int> f, pair <int, int> s) const {
    if (f == s) return false;
    if (s.first <= f.first && s.second <= f.second) {
      return true;
    }
    if (f.first <= s.first && f.second <= s.second) {
      return false;
    }
    return cmp(p[s.first], p[s.second], p[f.first], p[f.second]);
  };
};

vector <pair <int, int>> solve(int n) {
  p.resize(n);
  iota(p.begin(), p.end(), 0);
  stable_sort(p.begin(), p.end(), [&] (int i, int j) {
    return cmp(i, i, j, j);
  });
  int cnt = 0;
  for (int i = 1; i <= n; ++i) {
    if (!cmp(p[i - 1], p[i - 1], p[i], p[i])) {
      ++cnt;
    }
  }
  assert(cnt <= 5);
  priority_queue <pair <int, int>, vector <pair <int, int>>, ccmp> q;
  for (int i = 0; i < n; ++i) q.push({i, i});
  vector <pair <int, int>> ans;
  while (!q.empty()) {
    auto pr = q.top();
    q.pop();
    ans.push_back({p[pr.first], p[pr.second]});
    if (pr.second + 1 < n) q.push({pr.first, pr.second + 1});
  }
  return ans;
}

#ifdef LOCAL
static const int HARD_LIMIT = 50000000;

static int n;
static std::vector<int> a;

static bool valid_pos(int i) {
	return 0 <= i && i < n;
}

static int Q;
bool cmp(int a_, int b, int c, int d) {
	if(!valid_pos(a_) || !valid_pos(b) || !valid_pos(c) || !valid_pos(d)) {
		std::cerr << "Invalid query." << std::endl;
		std::cout << -1 << std::endl;
		exit(0);
	}

	Q++;
	if(Q > HARD_LIMIT) {
		std::cerr << "Too many queries." << std::endl;
		std::cout << -1 << std::endl;
		exit(0);
	}

	return a[a_] + a[b] < a[c] + a[d];
}

static bool check_valid(const std::vector<std::pair<int, int>> &a) {
	if(n * (n + 1) / 2 != (int)a.size()) {
		return false;
	}

	std::vector<std::vector<bool>> used(n, std::vector<bool>(n, false));
	for(int i = 0; i < (int)a.size(); i++) {
		if(!valid_pos(a[i].first) || !valid_pos(a[i].second) || used[a[i].first][a[i].second]) {
			return false;
		}

		used[a[i].first][a[i].second] = true;
		used[a[i].second][a[i].first] = true;
	}

	return true;
}

int main() {
  ios_base::sync_with_stdio(false); cin.tie(0);
#ifdef LOCAL
  freopen("input.txt", "r", stdin);
#endif
  //std::cin >> n;
  n = 2000;
  mt19937 rnd(123);
	a.resize(n);
	for(int i = 0; i < n; i++) {
		a[i] = rnd() % (int)1e9;
	}

	Q = 0;
	std::vector<std::pair<int, int>> ret = solve(n);	
	
	if(!check_valid(ret)) {
		std::cerr << "Invalid output." << std::endl;
		std::cout << -1 << std::endl;
		return 0;
	}

	for(int i = 1; i < (int) ret.size(); i++) { 
		if(a[ret[i].first] + a[ret[i].second] < a[ret[i - 1].first] + a[ret[i - 1].second]) {
			std::cerr << "Output not sorted." << std::endl;
			std::cout << -1 << std::endl;
			return 0;
		}
	}
	
	std::cerr << "Correct!" << std::endl;
	std::cout << Q << std::endl;
	return 0; 
}
#endif
