#include "sqsort.h"
#include <iostream>
#include <vector>
#include <string>
#include <set>
#include <map>
#include <algorithm>
#include <iomanip>
#include <chrono>
#include <utility>
#include <cmath>
#pragma GCC optimize("O3")
#pragma GCC target("sse,sse2,sse3,sse4")
//#define int long long
#define pii pair<int, int>
#define x first
#define y second
#define all(a) (a.begin()), (a.end())

using namespace std;

const int N = 2e3 + 10;

bool ls[N][N];
vector<pii> ranges;
int starts[N];
int endings[N];
vector<int> perm;

bool mycmp_spec(const pii &p1, const pii &p2)
{
	return cmp(p1.x, p1.y, p2.x, p2.y);
}

bool mycmp(const pii &p1, const pii &p2)
{
	if ((ls[p1.x][p2.x] && ls[p1.y][p2.y]) || (ls[p1.x][p2.y] && ls[p1.y][p2.x]))
		return true;

	if ((!ls[p1.x][p2.x] && !ls[p1.y][p2.y]) || (!ls[p1.x][p2.y] && !ls[p1.y][p2.x]))
		return false;

	return cmp(p1.x, p1.y, p2.x, p2.y);
}

void preproc(int n)
{
	vector<pii> v(n);
	perm.resize(n);

	for (int i = 0; i < n; ++i)
		perm[i] = i;

	for (int i = 0; i < n; ++i)
		v[i] = { i, i };

	stable_sort(all(perm), [&v](int i, int j) { return mycmp_spec(v[i], v[j]); });

	vector<pii> nv(n);
	
	for (int i = 0; i < n; ++i)
		nv[i] = v[perm[i]];

	copy(all(nv), v.begin());
	
	for(int i = 0; i < n; ++i)
		for (int j = i; j < n; ++j)
		{
			int ind1 = v[i].y;
			int ind2 = v[j].y;
			ls[ind1][ind2] = true;
			ls[ind2][ind1] = false;
		}
}

void mysort(vector<pii> &v, int l, int r)
{
	if (l == r)
		return;

	int mid = (l + r) / 2;

	mysort(v, l, mid);
	mysort(v, mid + 1, r);

	int strt = starts[l];
	int middle = endings[mid];
	int end = endings[r];

	inplace_merge(v.begin() + strt, v.begin() + middle, v.begin() + end, mycmp);
}

std::vector<std::pair<int, int>> solve(int n)
{
	preproc(n);
	vector<pii> rv;

	rv.reserve(n * (n + 1) / 2 + 10);

	for (int i = 0; i < n; ++i)
	{
		starts[i] = rv.size();

		for (int j = i; j < n; ++j)
		{
			rv.emplace_back(perm[i], perm[j]);

			if (rv.back().x > rv.back().y)
				swap(rv[rv.size() - 1].x, rv[rv.size() - 1].y);
		}

		endings[i] = rv.size();
	}

	mysort(rv, 0, n - 1);

	return move(rv);
}