#include <vector>
#include <algorithm>
#include <queue>

#include "sqsort.h"

using namespace std;

bool cmp_pos(int x, int y) {
    return cmp(x, x, y, y);
}

typedef vector<pair<int, int>> PV;

class cmp_size {
public:
    bool operator()(const PV& a, const PV& b) {
        return a.size() > b.size();
    }
};

PV merge_pv(const PV& a, const PV& b) {
    PV c;
    int i = 0, j = 0;
    while (i < a.size() && j < b.size()) {
        if (cmp(a[i].first, a[i].second, b[j].first, b[j].second)) {
            c.push_back(a[i]);
            i++;
        } else {
            c.push_back(b[j]);
            j++;
        }
    }
    while (i < a.size()) {
        c.push_back(a[i]);
        i++;
    }

    while (j < b.size()) {
        c.push_back(b[j]);
        j++;
    }
    return c;
}

PV solve(int N) {
    vector<int> sorted;
    for (int i = 0; i < N; i++)
        sorted.push_back(i);
    sort(sorted.begin(), sorted.end(), cmp_pos);
    priority_queue<PV, vector<PV>, cmp_size> q;
    for (int i = 0; i < N; i++) {
        PV a;
        for (int j : sorted)
            if (j >= i)
                a.emplace_back(i, j);
        q.push(a);
    }
    while (q.size() > 1) {
        PV a = q.top();
        q.pop();
        PV b = q.top();
        q.pop();

        q.push(merge_pv(a, b));
    }
    return q.top();
}
