#include <vector>
#include <iostream>
#include <algorithm>
#include <queue>

#include "sqsort.h"

using namespace std;

bool cmp_pos(int x, int y) {
    return cmp(x, x, y, y);
}

typedef vector<pair<int, int>> PV;
typedef pair<int, int>* PVC;

class cmp_size {
public:
    bool operator()(const PV& a, const PV& b) {
        return a.size() > b.size();
    }
};

void merge_pv(PVC a, PVC b, int na, int nb, PVC c) {
//    cerr << na << ' ' << nb << endl;
    if (na < nb) {
        swap(a, b);
        swap(na, nb);
    }

    if (!nb) {
        for (int i = 0; i < na; i++)
            c[i] = a[i];
        return;
    }
    if (!na) {
        for (int i = 0; i < nb; i++)
            c[i] = b[i];
        return;
    }

    pair<int, int> piv = a[na / 2];
    int st = -1, dr = nb - 1;
    while (st != dr) {
        int mid = (st + dr + 1) / 2;
        if (cmp(b[mid].first, b[mid].second, piv.first, piv.second))
            st = mid;
        else
            dr = mid - 1;
    }
//    PV a1, a2, b1, b2, c1, c2;
//    for (int i = 0; i < na / 2; i++)
//        a1.push_back(a[i]);
//    for (int i = na / 2 + 1; i < na; i++)
//        a2.push_back(a[i]);
//    for (int i = 0; i <= st; i++)
//        b1.push_back(b[i]);
//    for (int i = st + 1; i < nb; i++)
//        b2.push_back(b[i]);
//    c1 = merge_pv(a1, b1);
//    c2 = merge_pv(a2, b2);

    merge_pv(a, b, na / 2, st + 1, c);
    c[na / 2 + st + 1] = piv;
    merge_pv(a + na / 2 + 1, b + st + 1, na - na / 2 - 1, nb - st - 1, c + na / 2 + st + 2);
}

const int MAX_N = 2000;
pair<int, int> v1[MAX_N * MAX_N];
pair<int, int> v2[MAX_N * MAX_N];
pair<int, int> v3[MAX_N * MAX_N];

PV solve(int N) {
    vector<int> sorted;
    for (int i = 0; i < N; i++)
        sorted.push_back(i);
    sort(sorted.begin(), sorted.end(), cmp_pos);

    priority_queue<PV, vector<PV>, cmp_size> q;
    for (int i = 0; i < N; i++) {
        PV a;
        for (int j : sorted)
            if (j >= i)
                a.emplace_back(i, j);
        q.push(a);
    }
    while (q.size() > 1) {
        PV a = q.top();
        q.pop();
        PV b = q.top();
        q.pop();

        for (int i = 0; i < a.size(); i++)
            v1[i] = a[i];
        for (int i = 0; i < b.size(); i++)
            v2[i] = b[i];
        merge_pv(v1, v2, a.size(), b.size(), v3);
        PV c(a.size() + b.size());
        for (int i = 0; i < a.size() + b.size(); i++)
            c[i] = v3[i];
        q.push(c);
    }
    return q.top();
}
