#include "sqsort.h"
#include <iostream>
#include <algorithm>
#include <numeric>
#include <iterator>
#include <unordered_map>

#define IDX(v) std::distance(indexes.begin(), std::find(indexes.begin(), indexes.end(), v))

using pvec = std::vector<std::pair<int, int>>;

std::vector<int> indexes;
std::unordered_map<int, int> idx_map;

bool custom_cmp(int a, int b, int c, int d) {
    if (a == c) {
        return idx_map[b] < idx_map[d];
    }
    if (b == d) {
        return idx_map[a] < idx_map[c];
    }
    if (a == d) {
        return idx_map[b] < idx_map[c];
    }
    if (b == c) {
        return idx_map[a] < idx_map[d];
    }

    return cmp(a, b, c, d);
}

template <class Iter>
auto merge_sort(Iter a, Iter b) {
    auto dist = std::distance(a, b);
    if (dist <= 1) {
        return;
    }

    auto mid = std::next(a, dist / 2);

    merge_sort(a, mid);
    merge_sort(mid, b);

    std::inplace_merge(a, mid, b,
                       [&](const auto& x, const auto& y) {
                           return custom_cmp(x.first, x.second, y.first, y.second);
                       });
}

pvec solve(int n) {
    pvec res;

    indexes.resize(n);
    std::iota(indexes.begin(), indexes.end(), 0);

    std::sort(indexes.begin(), indexes.end(),
              [&](const auto& x, const auto& y) {
                  return cmp(x, x, y, y);
              });

    for (auto i = 0; i < n; ++i) {
        idx_map[indexes[i]] = i;
    }

    // Generate result array
    for (auto a = 0; a < n; ++a) {
        for (auto b = a; b < n; ++b) {
            res.emplace_back(a, b);
        }
    }

    // Sort result array
    /*std::sort(res.begin(), res.end(),
              [&](const auto& x, const auto& y) {
                  return custom_cmp(x.first, x.second, y.first, y.second);
              });*/

    merge_sort(res.begin(), res.end());

    return res;
}
