#include "sqsort.h"
#include <iostream>
#include <algorithm>
#include <numeric>
#include <iterator>
#include <unordered_map>
#include <iomanip>

using pvec = std::vector<std::pair<int, int>>;

std::vector<int> indexes;
std::unordered_map<int, int> idx_map;

bool custom_cmp(int a, int b, int c, int d) {
    if (a == c) {
        return idx_map[b] < idx_map[d];
    }
    if (b == d) {
        return idx_map[a] < idx_map[c];
    }
    if (a == d) {
        return idx_map[b] < idx_map[c];
    }
    if (b == c) {
        return idx_map[a] < idx_map[d];
    }

    auto m = idx_map[a];
    auto n = idx_map[b];
    if (m > n) {
        std::swap(m, n);
    }

    auto p = idx_map[c];
    auto q = idx_map[d];
    if (p > q) {
        std::swap(p, q);
    }

    auto x = m < p;
    auto y = n < q;

    if (x == y) {
        return x;
    }

    return cmp(a, b, c, d);
}

template <class Iter, class Comp>
auto merge_sort(Iter a, Iter b, Comp c) {
    auto dist = std::distance(a, b);
    if (dist <= 1) {
        return;
    }

    auto mid = std::next(a, dist / 2);

    merge_sort(a, mid, c);
    merge_sort(mid, b, c);

    std::inplace_merge(a, mid, b, c);
}

pvec solve(int n) {
    pvec res;

    indexes.resize(n);
    std::iota(indexes.begin(), indexes.end(), 0);

    std::sort(indexes.begin(), indexes.end(),
              [&](const auto& x, const auto& y) {
                  return cmp(x, x, y, y);
              });

    for (auto i = 0; i < n; ++i) {
        idx_map[indexes[i]] = i;
    }

    // Generate result array
    for (auto a = 0; a < n; ++a) {
        for (auto b = a; b < n; ++b) {
            res.emplace_back(a, b);
        }
    }

    merge_sort(res.begin(), res.end(),
               [&](const auto& x, const auto& y) {
                   return custom_cmp(x.first, x.second, y.first, y.second);
               });

    return res;
}
