#include <map>
#include <set>
#include <vector>
#include <iostream>
#include <functional>

#include "sqsort.h"
//#include "Lgrader.cpp"

using namespace std;

const int MAXN = 2005;

struct MyPair
{
    int i, j;

    MyPair(){}
    MyPair(int i, int j)
    {
        this->i = i;
        this->j = j;
    }
};

int numberSize[MAXN];

void initNumberSize(int n)
{
    vector <int> v;
    for(int i = 0;i<n;i++)
        v.push_back(i);

    sort(v.begin(), v.end(),
    [](int i, int j)
    {
        return cmp(i, i, j, j);
    });

    int ind = 0;
    for(int i = 0;i<n;i++)
    {
        if(i!=0 && cmp(v[i-1], v[i-1], v[i], v[i])==true) ind++;
        numberSize[ v[i] ] = ind;
    }

}

struct DSU
{
    int parent[500*500];

    DSU(){}
    DSU(int n)
    {
        for(int i = 0;i<=n;i++)
            this->parent[i] = i;
    }

    int Find(int x)
    {
        if(parent[x]==x) return x;

        parent[x] = Find(parent[x]);
        return parent[x];
    }

    void Union(int u, int v)
    {
        u = Find(u);
        v = Find(v);
        if(u==v) return;

        parent[v] = u;
    }
};

vector <vector <MyPair>> sortCluster;

void initClusters(int n)
{
    vector <MyPair> v;
    for(int i = 0;i<n;i++)
        for(int j = i;j<n;j++)
            v.push_back(MyPair(i, j));

    function <int(int, int)> encode =
    [&](int i, int j){ return i*n + j; };


    struct Element
    {
        int pos;
        bool type;

        int code;

        Element(){}
        Element(int pos, bool type, int code)
        {
            this->pos = pos;
            this->type = type;
            this->code = code;
        }
    };

    vector <Element> sweep;
    for(int i = 0;i<v.size();i++)
    {
        int l = numberSize[v[i].i];
        int r = numberSize[v[i].j];
        int code = encode(v[i].i, v[i].j);

        if(l>r) swap(l, r);
        sweep.push_back(Element(l, false, code));
        sweep.push_back(Element(r, true, code));
    }
    sort(sweep.begin(), sweep.end(),
    [](Element A, Element B)
    {
        if(A.pos!=B.pos) return A.pos<B.pos;
        return A.type>B.type;
    });

    DSU T(n*n+22);
    set <int> active;


    for(Element e: sweep)
    {
        if(e.type==false)
        {
            if(active.empty()==false)
            {
                T.Union(*active.begin(), e.code);
            }

            active.insert(e.code);
        }
        else
        {
            active.erase(e.code);
        }
    }

    map <int, vector <MyPair>> mp;
    for(MyPair item: v)
    {
        int code = encode(item.i, item.j);
        mp[T.Find(code)].push_back(item);
    }

    for(pair <int, vector <MyPair>> item: mp)
    {
        sortCluster.push_back(item.second);
    }
}

vector<pair<int, int>> solve(int n)
{

    initNumberSize(n);
    initClusters(n);

    vector <pair <int, int>> ans;
    for(vector <MyPair> &v: sortCluster)
    {
        sort(v.begin(), v.end(),
        [](MyPair A, MyPair B)
        {
            if(max(numberSize[A.i], numberSize[A.j])<=min(numberSize[B.i], numberSize[B.j])) return true;
            if(min(numberSize[A.i], numberSize[A.j])>=max(numberSize[B.i], numberSize[B.j])) return false;

            return cmp(A.i, A.j, B.i, B.j);
        });

        for(MyPair mp: v) ans.push_back({mp.i, mp.j});
    }

    return ans;
}

